@@ -1,7 +1,7 @@
 /*
  *  yosys -- Yosys Open SYnthesis Suite
  *
- *  Copyright (C) 2012  Clifford Wolf <clifford@clifford.at>
+ *  Copyright (C) 2012  Claire Xenia Wolf <claire@yosyshq.com>
  *
  *  Permission to use, copy, modify, and/or distribute this software for any
  *  purpose with or without fee is hereby granted, provided that the above
@@ -19,7 +19,7 @@
 
 #include "kernel/yosys.h"
 #include "libs/sha1/sha1.h"
-#include "backends/ilang/ilang_backend.h"
+#include "backends/rtlil/rtlil_backend.h"
 
 #if !defined(_WIN32) || defined(__MINGW32__)
 #  include <sys/time.h>
@@ -41,9 +41,12 @@ YOSYS_NAMESPACE_BEGIN
 std::vector<FILE*> log_files;
 std::vector<std::ostream*> log_streams;
 std::map<std::string, std::set<std::string>> log_hdump;
-std::vector<std::regex> log_warn_regexes, log_nowarn_regexes, log_werror_regexes;
-std::set<std::string> log_warnings;
+std::vector<YS_REGEX_TYPE> log_warn_regexes, log_nowarn_regexes, log_werror_regexes;
+dict<std::string, LogExpectedItem> log_expect_log, log_expect_warning, log_expect_error;
+std::set<std::string> log_warnings, log_experimentals, log_experimentals_ignored;
 int log_warnings_count = 0;
+int log_warnings_count_noexpect = 0;
+bool log_expect_no_warnings = false;
 bool log_hdump_all = false;
 FILE *log_errfile = NULL;
 SHA1 *log_hasher = NULL;
@@ -61,7 +64,7 @@ int log_force_debug = 0;
 int log_debug_suppressed = 0;
 
 vector<int> header_count;
-pool<RTLIL::IdString> log_id_cache;
+vector<char*> log_id_cache;
 vector<shared_str> string_buf;
 int string_buf_index = -1;
 
@@ -69,6 +72,13 @@ static struct timeval initial_tv = { 0, 0 };
 static bool next_print_log = false;
 static int log_newline_count = 0;
 
+static void log_id_cache_clear()
+{
+	for (auto p : log_id_cache)
+		free(p);
+	log_id_cache.clear();
+}
+
 #if defined(_WIN32) && !defined(__MINGW32__)
 // this will get time information and return it in timeval, simulating gettimeofday()
 int gettimeofday(struct timeval *tv, struct timezone *tz)
@@ -155,7 +165,7 @@ void logv(const char *format, va_list ap)
 	{
 		log_warn_regex_recusion_guard = true;
 
-		if (log_warn_regexes.empty())
+		if (log_warn_regexes.empty() && log_expect_log.empty())
 		{
 			linebuffer.clear();
 		}
@@ -165,8 +175,13 @@ void logv(const char *format, va_list ap)
 
 			if (!linebuffer.empty() && linebuffer.back() == '\n') {
 				for (auto &re : log_warn_regexes)
-					if (std::regex_search(linebuffer, re))
+					if (YS_REGEX_NS::regex_search(linebuffer, re))
 						log_warning("Found log message matching -W regex:\n%s", str.c_str());
+
+				for (auto &item : log_expect_log)
+					if (YS_REGEX_NS::regex_search(linebuffer, item.second.pattern))
+						item.second.current_count++;
+
 				linebuffer.clear();
 			}
 		}
@@ -221,7 +236,7 @@ static void logv_warning_with_prefix(const char *prefix,
 	bool suppressed = false;
 
 	for (auto &re : log_nowarn_regexes)
-		if (std::regex_search(message, re))
+		if (YS_REGEX_NS::regex_search(message, re))
 			suppressed = true;
 
 	if (suppressed)
@@ -234,9 +249,16 @@ static void logv_warning_with_prefix(const char *prefix,
 		log_make_debug = 0;
 
 		for (auto &re : log_werror_regexes)
-			if (std::regex_search(message, re))
+			if (YS_REGEX_NS::regex_search(message, re))
 				log_error("%s",  message.c_str());
 
+		bool warning_match = false;
+		for (auto &item : log_expect_warning)
+			if (YS_REGEX_NS::regex_search(message, item.second.pattern)) {
+				item.second.current_count++;
+				warning_match = true;
+			}
+
 		if (log_warnings.count(message))
 		{
 			log("%s%s", prefix, message.c_str());
@@ -256,6 +278,8 @@ static void logv_warning_with_prefix(const char *prefix,
 			log_warnings.insert(message);
 		}
 
+		if (!warning_match)
+			log_warnings_count_noexpect++;
 		log_warnings_count++;
 		log_make_debug = bak_log_make_debug;
 	}
@@ -293,7 +317,7 @@ void log_file_info(const std::string &filename, int lineno,
 	va_end(ap);
 }
 
-YS_ATTRIBUTE(noreturn)
+[[noreturn]]
 static void logv_error_with_prefix(const char *prefix,
                                    const char *format, va_list ap)
 {
@@ -318,9 +342,17 @@ static void logv_error_with_prefix(const char *prefix,
 
 	log_make_debug = bak_log_make_debug;
 
+	for (auto &item : log_expect_error)
+		if (YS_REGEX_NS::regex_search(log_last_error, item.second.pattern))
+			item.second.current_count++;
+
+	log_check_expected();
+
 	if (log_error_atexit)
 		log_error_atexit();
 
+	YS_DEBUGTRAP_IF_DEBUGGING;
+
 #ifdef EMSCRIPTEN
 	log_files = backup_log_files;
 	throw 0;
@@ -370,6 +402,19 @@ void log_warning(const char *format, ...)
 	va_end(ap);
 }
 
+void log_experimental(const char *format, ...)
+{
+	va_list ap;
+	va_start(ap, format);
+	string s = vstringf(format, ap);
+	va_end(ap);
+
+	if (log_experimentals_ignored.count(s) == 0 && log_experimentals.count(s) == 0) {
+		log_warning("Feature '%s' is experimental.\n", s.c_str());
+		log_experimentals.insert(s);
+	}
+}
+
 void log_warning_noprefix(const char *format, ...)
 {
 	va_list ap;
@@ -414,7 +459,7 @@ void log_push()
 void log_pop()
 {
 	header_count.pop_back();
-	log_id_cache.clear();
+	log_id_cache_clear();
 	string_buf.clear();
 	string_buf_index = -1;
 	log_flush();
@@ -521,7 +566,7 @@ void log_reset_stack()
 {
 	while (header_count.size() > 1)
 		header_count.pop_back();
-	log_id_cache.clear();
+	log_id_cache_clear();
 	string_buf.clear();
 	string_buf_index = -1;
 	log_flush();
@@ -544,10 +589,14 @@ void log_dump_val_worker(RTLIL::SigSpec v) {
 	log("%s", log_signal(v));
 }
 
+void log_dump_val_worker(RTLIL::State v) {
+	log("%s", log_signal(v));
+}
+
 const char *log_signal(const RTLIL::SigSpec &sig, bool autoint)
 {
 	std::stringstream buf;
-	ILANG_BACKEND::dump_sigspec(buf, sig, autoint);
+	RTLIL_BACKEND::dump_sigspec(buf, sig, autoint);
 
 	if (string_buf.size() < 100) {
 		string_buf.push_back(buf.str());
@@ -580,8 +629,8 @@ const char *log_const(const RTLIL::Const &value, bool autoint)
 
 const char *log_id(RTLIL::IdString str)
 {
-	log_id_cache.insert(str);
-	const char *p = str.c_str();
+	log_id_cache.push_back(strdup(str.c_str()));
+	const char *p = log_id_cache.back();
 	if (p[0] != '\\')
 		return p;
 	if (p[1] == '$' || p[1] == '\\' || p[1] == 0)
@@ -594,24 +643,74 @@ const char *log_id(RTLIL::IdString str)
 void log_module(RTLIL::Module *module, std::string indent)
 {
 	std::stringstream buf;
-	ILANG_BACKEND::dump_module(buf, indent, module, module->design, false);
+	RTLIL_BACKEND::dump_module(buf, indent, module, module->design, false);
 	log("%s", buf.str().c_str());
 }
 
 void log_cell(RTLIL::Cell *cell, std::string indent)
 {
 	std::stringstream buf;
-	ILANG_BACKEND::dump_cell(buf, indent, cell);
+	RTLIL_BACKEND::dump_cell(buf, indent, cell);
 	log("%s", buf.str().c_str());
 }
 
 void log_wire(RTLIL::Wire *wire, std::string indent)
 {
 	std::stringstream buf;
-	ILANG_BACKEND::dump_wire(buf, indent, wire);
+	RTLIL_BACKEND::dump_wire(buf, indent, wire);
 	log("%s", buf.str().c_str());
 }
 
+void log_check_expected()
+{
+	// copy out all of the expected logs so that they cannot be re-checked
+	// or match against themselves
+	dict<std::string, LogExpectedItem> expect_log, expect_warning, expect_error;
+	std::swap(expect_warning, log_expect_warning);
+	std::swap(expect_log, log_expect_log);
+	std::swap(expect_error, log_expect_error);
+
+	for (auto &item : expect_warning) {
+		if (item.second.current_count == 0) {
+			log_warn_regexes.clear();
+			log_error("Expected warning pattern '%s' not found !\n", item.first.c_str());
+		}
+		if (item.second.current_count != item.second.expected_count) {
+			log_warn_regexes.clear();
+			log_error("Expected warning pattern '%s' found %d time(s), instead of %d time(s) !\n",
+				item.first.c_str(), item.second.current_count, item.second.expected_count);
+		}
+	}
+
+	for (auto &item : expect_log) {
+		if (item.second.current_count == 0) {
+			log_warn_regexes.clear();
+			log_error("Expected log pattern '%s' not found !\n", item.first.c_str());
+		}
+		if (item.second.current_count != item.second.expected_count) {
+			log_warn_regexes.clear();
+			log_error("Expected log pattern '%s' found %d time(s), instead of %d time(s) !\n",
+				item.first.c_str(), item.second.current_count, item.second.expected_count);
+		}
+	}
+
+	for (auto &item : expect_error)
+		if (item.second.current_count == item.second.expected_count) {
+			log_warn_regexes.clear();
+			log("Expected error pattern '%s' found !!!\n", item.first.c_str());
+			#ifdef EMSCRIPTEN
+				throw 0;
+			#elif defined(_MSC_VER)
+				_exit(0);
+			#else
+				_Exit(0);
+			#endif
+		} else {
+			log_warn_regexes.clear();
+			log_error("Expected error pattern '%s' not found !\n", item.first.c_str());
+		}
+}
+
 // ---------------------------------------------------
 // This is the magic behind the code coverage counters
 // ---------------------------------------------------
